@@ -1,17 +1,18 @@
-# -*- coding: iso-8859-1 -*-
+# -*- coding: utf-8 -*-
+
 """
 Trac
 Edgewall Software
 
-U{http://trac.edgewall.com/}
+U{http://trac.edgewall.org/}
 
-@author: Jonas Borgstrm <jonas@edgewall.com>
+@author: Jonas Borgström <jonas@edgewall.com>
 @author: Daniel Lundin <daniel@edgewall.com>
 """
 __docformat__ = 'epytext en'
 
-__version__ = '0.9.6'
-__url__ = 'http://trac.edgewall.com/'
+__version__ = '0.10.1'
+__url__ = 'http://trac.edgewall.org/'
 __copyright__ = '(C) 2003-2006 Edgewall Software'
 __license__ = 'BSD'
 __license_long__ = """
@@ -43,3 +44,4 @@ __license_long__ = """
  IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
  IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."""
+
