@@ -1,38 +1,31 @@
-# -*- coding: iso-8859-1 -*-
+# -*- coding: utf-8 -*-
 # 
-# Copyright (C) 2004-2005 Edgewall Software
+# Copyright (C) 2004-2006 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.com/license.html.
+# are also available at http://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://projects.edgewall.com/trac/.
+# history and logs, available at http://trac.edgewall.org/log/.
 #
 # Author: Tim Moloney <t.moloney@verizon.net>
 
-
-from trac.db_default import data as default_data, default_config
-from trac.config import Configuration
-from trac.env import Environment
-from trac.scripts import admin
-from trac.test import InMemoryDatabase
-from trac.util import get_date_format_hint, NaivePopen
-
+import ConfigParser
 import os
 import re
+import shlex
 import sys
 import time
 import unittest
-import shlex
-import ConfigParser
+from StringIO import StringIO
 
-try:
-    from cStringIO import StringIO
-except ImportError:
-    from StringIO import StringIO
+from trac.env import Environment
+from trac.scripts import admin
+from trac.test import InMemoryDatabase, TestConfiguration
+from trac.util.datefmt import get_date_format_hint
 
 STRIP_TRAILING_SPACE = re.compile(r'( +)$', re.MULTILINE)
 
@@ -48,7 +41,7 @@ def load_expected_results(file, pattern):
     compiled_pattern = re.compile(pattern)
     f = open(file, 'r')
     for line in f:
-        line = line.rstrip()
+        line = line.rstrip().decode('utf-8')
         match = compiled_pattern.search(line)
         if match:
             test = match.groups()[0]
@@ -59,10 +52,10 @@ def load_expected_results(file, pattern):
     return expected
 
 
-"""
-A subclass of Environment that keeps its' DB in memory.
-"""
 class InMemoryEnvironment(Environment):
+    """
+    A subclass of Environment that keeps its' DB in memory.
+    """
 
     def get_db_cnx(self):
         if not hasattr(self, '_db'):
@@ -70,7 +63,7 @@ class InMemoryEnvironment(Environment):
         return self._db
 
     def create(self, db_str=None):
-        self.load_config()
+        pass
 
     def verify(self):
         return True
@@ -83,10 +76,8 @@ class InMemoryEnvironment(Environment):
         return cls.__module__.startswith('trac.') and \
                cls.__module__.find('.tests.') == -1
 
-    def load_config(self):
-        self.config = Configuration(None)
-        for section, name, value in default_config:
-            self.config.setdefault(section, name, value)
+    def setup_config(self, load_defaults=None):
+        self.config = TestConfiguration(None)
             
     def save_config(self):
         pass
@@ -125,15 +116,19 @@ class TracadminTestCase(unittest.TestCase):
             _err = sys.stderr
             _out = sys.stdout
             sys.stderr = sys.stdout = out = StringIO()
+            setattr(out, 'encoding', 'utf-8') # fake output encoding
             retval = None
             try:
                 retval = self._admin.onecmd(cmd)
             except SystemExit, e:
                 pass
+            value = out.getvalue()
+            if isinstance(value, str): # reverse what print_listing did
+                value = value.decode('utf-8')
             if strip_trailing_space:
-                return retval, STRIP_TRAILING_SPACE.sub('', out.getvalue())
+                return retval, STRIP_TRAILING_SPACE.sub('', value)
             else:
-                return retval, out.getvalue()
+                return retval, value
         finally:
             sys.stderr = _err
             sys.stdout = _out
@@ -505,6 +500,18 @@ Trac Admin Console %s
         self.assertEqual(0, rv)
         self.assertEqual(self.expected_results[test_name], output)
 
+    def test_priority_add_many_ok(self):
+        """
+        Tests adding more than 10 priority values.  This makes sure that
+        ordering is preserved when adding more than 10 values.
+        """
+        test_name = sys._getframe().f_code.co_name
+        for i in xrange(11):
+            self._execute('priority add p%s' % i)
+        rv, output = self._execute('priority list')
+        self.assertEqual(0, rv)
+        self.assertEqual(self.expected_results[test_name], output)
+
     def test_priority_add_error_already_exists(self):
         """
         Tests the 'priority add' command in trac-admin.  This particular
@@ -850,6 +857,18 @@ Trac Admin Console %s
         self.assertEqual(0, rv)
         self.assertEqual(self.expected_results[test_name], output)
 
+    def test_milestone_add_utf8_ok(self):
+        """
+        Tests the 'milestone add' command in trac-admin.  This particular
+        test passes valid arguments and checks for success.
+        """
+        test_name = sys._getframe().f_code.co_name
+        self._execute(u'milestone add \xe9tat_final "%s"'  #\xc3\xa9
+                              % self._test_date)
+        rv, output = self._execute('milestone list')
+        self.assertEqual(0, rv)
+        self.assertEqual(self.expected_results[test_name], output)
+
     def test_milestone_add_error_already_exists(self):
         """
         Tests the 'milestone add' command in trac-admin.  This particular
@@ -950,6 +969,13 @@ Trac Admin Console %s
         self.assertEqual(2, rv)
         self.assertEqual(self.expected_results[test_name], output)
 
+    def test_backslash_use_ok(self):
+        test_name = sys._getframe().f_code.co_name
+        self._execute('version add \\')
+        rv, output = self._execute('version list')
+        self.assertEqual(0, rv)
+        self.assertEqual(self.expected_results[test_name], output)
+
 
 def suite():
     return unittest.makeSuite(TracadminTestCase, 'test')
