@@ -1,25 +1,27 @@
-# -*- coding: iso-8859-1 -*-
+# -*- coding: utf-8 -*-
 #
 # Copyright (C) 2003-2006 Edgewall Software
-# Copyright (C) 2003-2006 Jonas Borgstrm <jonas@edgewall.com>
+# Copyright (C) 2003-2006 Jonas Borgström <jonas@edgewall.com>
 # Copyright (C) 2005 Christopher Lenz <cmlenz@gmx.de>
 # Copyright (C) 2006 Christian Boos <cboos@neuf.fr>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.com/license.html.
+# are also available at http://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://projects.edgewall.com/trac/.
+# history and logs, available at http://trac.edgewall.org/log/.
 #
-# Author: Jonas Borgstrm <jonas@edgewall.com>
+# Author: Jonas Borgström <jonas@edgewall.com>
 #         Christopher Lenz <cmlenz@gmx.de>
 
-import time
+import re
 import sys
+import time
 
+from trac.attachment import Attachment
 from trac.core import TracError
 from trac.ticket import TicketSystem
 from trac.util import sorted, embedded_numbers
@@ -34,13 +36,22 @@ class Ticket(object):
         self.env = env
         self.fields = TicketSystem(self.env).get_ticket_fields()
         self.values = {}
-        if tkt_id:
+        if tkt_id is not None:
             self._fetch_ticket(tkt_id, db)
         else:
             self._init_defaults(db)
             self.id = self.time_created = self.time_changed = None
         self._old = {}
 
+    def _get_db(self, db):
+        return db or self.env.get_db_cnx()
+
+    def _get_db_for_write(self, db):
+        if db:
+            return (db, False)
+        else:
+            return (self.env.get_db_cnx(), True)
+
     exists = property(fget=lambda self: self.id is not None)
 
     def _init_defaults(self, db=None):
@@ -66,8 +77,7 @@ class Ticket(object):
                 self.values.setdefault(field['name'], default)
 
     def _fetch_ticket(self, tkt_id, db=None):
-        if not db:
-            db = self.env.get_db_cnx()
+        db = self._get_db(db)
 
         # Fetch the standard ticket fields
         std_fields = [f['name'] for f in self.fields if not f.get('custom')]
@@ -125,15 +135,12 @@ class Ticket(object):
     def insert(self, when=0, db=None):
         """Add ticket to database"""
         assert not self.exists, 'Cannot insert an existing ticket'
-        if not db:
-            db = self.env.get_db_cnx()
-            handle_ta = True
-        else:
-            handle_ta = False
+        db, handle_ta = self._get_db_for_write(db)
 
         # Add a timestamp
         if not when:
-            when = int(time.time())
+            when = time.time()
+        when = int(when)
         self.time_created = self.time_changed = when
 
         cursor = db.cursor()
@@ -168,11 +175,16 @@ class Ticket(object):
 
         if handle_ta:
             db.commit()
+
         self.id = tkt_id
         self._old = {}
+
+        for listener in TicketSystem(self.env).change_listeners:
+            listener.ticket_created(self)
+
         return self.id
 
-    def save_changes(self, author, comment, when=0, db=None):
+    def save_changes(self, author, comment, when=0, db=None, cnum=''):
         """
         Store ticket changes in the database. The ticket must already exist in
         the database.
@@ -182,11 +194,7 @@ class Ticket(object):
         if not self._old and not comment:
             return # Not modified
 
-        if not db:
-            db = self.env.get_db_cnx()
-            handle_ta = True
-        else:
-            handle_ta = False
+        db, handle_ta = self._get_db_for_write(db)
         cursor = db.cursor()
         when = int(when or time.time())
 
@@ -208,6 +216,14 @@ class Ticket(object):
                     # just leave the owner as is.
                     pass
 
+        # Fix up cc list separators and remove duplicates
+        if self.values.has_key('cc'):
+            cclist = []
+            for cc in re.split(r'[;,\s]+', self.values['cc']):
+                if cc not in cclist:
+                    cclist.append(cc)
+            self.values['cc'] = ', '.join(cclist)
+
         custom_fields = [f['name'] for f in self.fields if f.get('custom')]
         for name in self._old.keys():
             if name in custom_fields:
@@ -229,52 +245,76 @@ class Ticket(object):
                            "VALUES (%s, %s, %s, %s, %s, %s)",
                            (self.id, when, author, name, self._old[name],
                             self[name]))
-        if comment:
-            cursor.execute("INSERT INTO ticket_change "
-                           "(ticket,time,author,field,oldvalue,newvalue) "
-                           "VALUES (%s,%s,%s,'comment','',%s)",
-                           (self.id, when, author, comment))
+        # always save comment, even if empty (numbering support for timeline)
+        cursor.execute("INSERT INTO ticket_change "
+                       "(ticket,time,author,field,oldvalue,newvalue) "
+                       "VALUES (%s,%s,%s,'comment',%s,%s)",
+                       (self.id, when, author, cnum, comment))
 
         cursor.execute("UPDATE ticket SET changetime=%s WHERE id=%s",
                        (when, self.id))
+
         if handle_ta:
             db.commit()
+        old_values = self._old
         self._old = {}
         self.time_changed = when
 
+        for listener in TicketSystem(self.env).change_listeners:
+            listener.ticket_changed(self, comment, author, old_values)
+
     def get_changelog(self, when=0, db=None):
         """Return the changelog as a list of tuples of the form
-        (time, author, field, oldvalue, newvalue).
+        (time, author, field, oldvalue, newvalue, permanent).
+
+        While the other tuple elements are quite self-explanatory,
+        the `permanent` flag is used to distinguish collateral changes
+        that are not yet immutable (like attachments, currently).
         """
-        if not db:
-            db = self.env.get_db_cnx()
+        when = int(when)
+        db = self._get_db(db)
         cursor = db.cursor()
         if when:
-            cursor.execute("SELECT time,author,field,oldvalue,newvalue "
+            cursor.execute("SELECT time,author,field,oldvalue,newvalue,1 "
                            "FROM ticket_change WHERE ticket=%s AND time=%s "
                            "UNION "
-                           "SELECT time,author,'attachment',null,filename "
+                           "SELECT time,author,'attachment',null,filename,0 "
                            "FROM attachment WHERE id=%s AND time=%s "
                            "UNION "
-                           "SELECT time,author,'comment',null,description "
+                           "SELECT time,author,'comment',null,description,0 "
                            "FROM attachment WHERE id=%s AND time=%s "
                            "ORDER BY time",
                            (self.id, when, str(self.id), when, self.id, when))
         else:
-            cursor.execute("SELECT time,author,field,oldvalue,newvalue "
+            cursor.execute("SELECT time,author,field,oldvalue,newvalue,1 "
                            "FROM ticket_change WHERE ticket=%s "
                            "UNION "
-                           "SELECT time,author,'attachment',null,filename "
+                           "SELECT time,author,'attachment',null,filename,0 "
                            "FROM attachment WHERE id=%s "
                            "UNION "
-                           "SELECT time,author,'comment',null,description "
+                           "SELECT time,author,'comment',null,description,0 "
                            "FROM attachment WHERE id=%s "
                            "ORDER BY time", (self.id,  str(self.id), self.id))
         log = []
-        for t, author, field, oldvalue, newvalue in cursor:
-            log.append((int(t), author, field, oldvalue or '', newvalue or ''))
+        for t, author, field, oldvalue, newvalue, permanent in cursor:
+            log.append((int(t), author, field, oldvalue or '', newvalue or '',
+                        permanent))
         return log
 
+    def delete(self, db=None):
+        db, handle_ta = self._get_db_for_write(db)
+        Attachment.delete_all(self.env, 'ticket', self.id, db)
+        cursor = db.cursor()
+        cursor.execute("DELETE FROM ticket WHERE id=%s", (self.id,))
+        cursor.execute("DELETE FROM ticket_change WHERE ticket=%s", (self.id,))
+        cursor.execute("DELETE FROM ticket_custom WHERE ticket=%s", (self.id,))
+
+        if handle_ta:
+            db.commit()
+
+        for listener in TicketSystem(self.env).change_listeners:
+            listener.ticket_deleted(self)
+
 
 class AbstractEnum(object):
     type = None
@@ -375,7 +415,8 @@ class AbstractEnum(object):
             db = env.get_db_cnx()
         cursor = db.cursor()
         cursor.execute("SELECT name,value FROM enum WHERE type=%s "
-                       "ORDER BY value", (cls.type,))
+                       "ORDER BY " + db.cast('value', 'int'),
+                       (cls.type,))
         for name, value in cursor:
             obj = cls(env)
             obj.name = obj._old_name = name
