@@ -1,4 +1,4 @@
-# -*- coding: iso-8859-1 -*-
+# -*- coding: utf-8 -*-
 #
 # Copyright (C) 2004-2006 Edgewall Software
 # Copyright (C) 2004-2005 Christopher Lenz <cmlenz@gmx.de>
@@ -7,23 +7,24 @@
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.com/license.html.
+# are also available at http://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://projects.edgewall.com/trac/.
+# history and logs, available at http://trac.edgewall.org/log/.
 #
 # Author: Christopher Lenz <cmlenz@gmx.de>
 
-from __future__ import generators
 import re
 from time import localtime, strftime, time
 
 from trac import __version__
 from trac.core import *
 from trac.perm import IPermissionRequestor
-from trac.util import enum, escape, format_date, format_datetime, parse_date, \
-                      pretty_timedelta, shorten_line, unescape, CRLF, Markup
+from trac.util.datefmt import format_date, format_datetime, parse_date, \
+                               pretty_timedelta
+from trac.util.html import html, unescape, Markup
+from trac.util.text import shorten_line, CRLF, to_unicode
 from trac.ticket import Milestone, Ticket, TicketSystem
 from trac.Timeline import ITimelineEventProvider
 from trac.web import IRequestHandler
@@ -46,22 +47,22 @@ def get_tickets_for_milestone(env, db, milestone, field='component'):
         tickets.append({'id': tkt_id, 'status': status, field: fieldval})
     return tickets
 
-def get_query_links(env, milestone, grouped_by='component', group=None):
+def get_query_links(req, milestone, grouped_by='component', group=None):
     q = {}
     if not group:
-        q['all_tickets'] = env.href.query(milestone=milestone)
-        q['active_tickets'] = env.href.query(milestone=milestone,
-                                             status=('new', 'assigned', 'reopened'))
-        q['closed_tickets'] = env.href.query(milestone=milestone, status='closed')
+        q['all_tickets'] = req.href.query(milestone=milestone)
+        q['active_tickets'] = req.href.query(
+            milestone=milestone, status=('new', 'assigned', 'reopened'))
+        q['closed_tickets'] = req.href.query(
+            milestone=milestone, status='closed')
     else:
-        q['all_tickets'] = env.href.query({grouped_by: group},
-                                          milestone=milestone)
-        q['active_tickets'] = env.href.query({grouped_by: group},
-                                             milestone=milestone,
-                                             status=('new', 'assigned', 'reopened'))
-        q['closed_tickets'] = env.href.query({grouped_by: group},
-                                             milestone=milestone,
-                                             status='closed')
+        q['all_tickets'] = req.href.query(
+            {grouped_by: group}, milestone=milestone)
+        q['active_tickets'] = req.href.query(
+            {grouped_by: group}, milestone=milestone,
+            status=('new', 'assigned', 'reopened'))
+        q['closed_tickets'] = req.href.query(
+            {grouped_by: group}, milestone=milestone, status='closed')
     return q
 
 def calc_ticket_stats(tickets):
@@ -90,7 +91,7 @@ def milestone_to_hdf(env, db, req, milestone):
     if milestone.exists:
         safe_name = milestone.name.replace('/', '%2F')
     hdf = {'name': milestone.name,
-           'href': env.href.milestone(safe_name)}
+           'href': req.href.milestone(safe_name)}
     if milestone.description:
         hdf['description_source'] = milestone.description
         hdf['description'] = wiki_to_html(milestone.description, env, req, db)
@@ -131,8 +132,7 @@ class RoadmapModule(Component):
         if not req.perm.has_permission('ROADMAP_VIEW'):
             return
         yield ('mainnav', 'roadmap',
-               Markup('<a href="%s" accesskey="3">Roadmap</a>',
-                      self.env.href.roadmap()))
+               html.a('Roadmap', href=req.href.roadmap(), accesskey=3))
 
     # IPermissionRequestor methods
 
@@ -156,13 +156,13 @@ class RoadmapModule(Component):
                       for m in Milestone.select(self.env, showall, db)]
         req.hdf['roadmap.milestones'] = milestones        
 
-        for idx,milestone in enum(milestones):
+        for idx, milestone in enumerate(milestones):
             milestone_name = unescape(milestone['name']) # Kludge
             prefix = 'roadmap.milestones.%d.' % idx
             tickets = get_tickets_for_milestone(self.env, db, milestone_name,
                                                 'owner')
             req.hdf[prefix + 'stats'] = calc_ticket_stats(tickets)
-            for k, v in get_query_links(self.env, milestone_name).items():
+            for k, v in get_query_links(req, milestone_name).items():
                 req.hdf[prefix + 'queries.' + k] = v
             milestone['tickets'] = tickets # for the iCalendar view
 
@@ -176,7 +176,7 @@ class RoadmapModule(Component):
         username = None
         if req.authname and req.authname != 'anonymous':
             username = req.authname
-        icshref = self.env.href.roadmap(show=req.args.get('show'),
+        icshref = req.href.roadmap(show=req.args.get('show'),
                                         user=username, format='ics')
         add_link(req, 'alternate', icshref, 'iCalendar', 'text/calendar', 'ics')
 
@@ -209,9 +209,13 @@ class RoadmapModule(Component):
                 else: return 'CANCELLED'
             else: return ''
 
+        def escape_value(text): 
+            s = ''.join(map(lambda c: (c in ';,\\') and '\\' + c or c, text))
+            return '\\n'.join(re.split(r'[\r\n]+', s))
+
         def write_prop(name, value, params={}):
             text = ';'.join([name] + [k + '=' + v for k, v in params.items()]) \
-                 + ':' + '\\n'.join(re.split(r'[\r\n]+', value))
+                 + ':' + escape_value(value)
             firstline = 1
             while text:
                 if not firstline: text = ' ' + text
@@ -237,8 +241,8 @@ class RoadmapModule(Component):
         write_prop('X-WR-CALNAME',
                    self.config.get('project', 'name') + ' - Roadmap')
         for milestone in milestones:
-            uid = '<%s/milestone/%s@%s>' % (req.cgi_location,
-                                            milestone['name'], host)
+            uid = '<%s/milestone/%s@%s>' % (req.base_path, milestone['name'],
+                                            host)
             if milestone.has_key('due'):
                 write_prop('BEGIN', 'VEVENT')
                 write_prop('UID', uid)
@@ -254,16 +258,18 @@ class RoadmapModule(Component):
                            if ticket['owner'] == user]:
                 ticket = Ticket(self.env, tkt_id)
                 write_prop('BEGIN', 'VTODO')
+                write_prop('UID', '<%s/ticket/%s@%s>' % (req.base_path,
+                                                         tkt_id, host))
                 if milestone.has_key('due'):
                     write_prop('RELATED-TO', uid)
                     write_date('DUE', localtime(milestone['due']))
                 write_prop('SUMMARY', 'Ticket #%i: %s' % (ticket.id,
                                                           ticket['summary']))
-                write_prop('URL', self.env.abs_href.ticket(ticket.id))
+                write_prop('URL', req.abs_href.ticket(ticket.id))
                 write_prop('DESCRIPTION', ticket['description'])
                 priority = get_priority(ticket)
                 if priority:
-                    write_prop('PRIORITY', str(priority))
+                    write_prop('PRIORITY', unicode(priority))
                 write_prop('STATUS', get_status(ticket))
                 if ticket['status'] == 'closed':
                     cursor = db.cursor()
@@ -316,11 +322,11 @@ class MilestoneModule(Component):
             for completed, name, description in cursor:
                 title = Markup('Milestone <em>%s</em> completed', name)
                 if format == 'rss':
-                    href = self.env.abs_href.milestone(name)
-                    message = wiki_to_html(description, self.env, db,
+                    href = req.abs_href.milestone(name)
+                    message = wiki_to_html(description, self.env, req, db,
                                            absurls=True)
                 else:
-                    href = self.env.href.milestone(name)
+                    href = req.href.milestone(name)
                     message = wiki_to_oneliner(description, self.env, db,
                                                shorten=True)
                 yield 'milestone', href, title, completed, None, message or '--'
@@ -332,13 +338,13 @@ class MilestoneModule(Component):
         match = re.match(r'/milestone(?:/(.+))?', req.path_info)
         if match:
             if match.group(1):
-                req.args['id'] = urllib.unquote(match.group(1))
+                req.args['id'] = match.group(1)
             return True
 
     def process_request(self, req):
         req.perm.assert_permission('MILESTONE_VIEW')
 
-        add_link(req, 'up', self.env.href.roadmap(), 'Roadmap')
+        add_link(req, 'up', req.href.roadmap(), 'Roadmap')
 
         db = self.env.get_db_cnx()
         milestone = Milestone(self.env, req.args.get('id'), db)
@@ -348,9 +354,9 @@ class MilestoneModule(Component):
             if req.args.has_key('cancel'):
                 if milestone.exists:
                     safe_name = milestone.name.replace('/', '%2F')
-                    req.redirect(self.env.href.milestone(safe_name))
+                    req.redirect(req.href.milestone(safe_name))
                 else:
-                    req.redirect(self.env.href.roadmap())
+                    req.redirect(req.href.roadmap())
             elif action == 'edit':
                 self._do_save(req, db, milestone)
             elif action == 'delete':
@@ -375,7 +381,7 @@ class MilestoneModule(Component):
             retarget_to = req.args.get('target')
         milestone.delete(retarget_to, req.authname)
         db.commit()
-        req.redirect(self.env.href.roadmap())
+        req.redirect(req.href.roadmap())
 
     def _do_save(self, req, db, milestone):
         if milestone.exists:
@@ -386,25 +392,35 @@ class MilestoneModule(Component):
         if not req.args.has_key('name'):
             raise TracError('You must provide a name for the milestone.',
                             'Required Field Missing')
-        milestone.name = req.args.get('name')
 
         due = req.args.get('duedate', '')
         try:
             milestone.due = due and parse_date(due) or 0
         except ValueError, e:
-            raise TracError(e, 'Invalid Date Format')
+            raise TracError(to_unicode(e), 'Invalid Date Format')
         if req.args.has_key('completed'):
             completed = req.args.get('completeddate', '')
             try:
                 milestone.completed = completed and parse_date(completed) or 0
             except ValueError, e:
-                raise TracError(e, 'Invalid Date Format')
+                raise TracError(to_unicode(e), 'Invalid Date Format')
             if milestone.completed > time():
                 raise TracError('Completion date may not be in the future',
                                 'Invalid Completion Date')
+            retarget_to = req.args.get('target')
+            if req.args.has_key('retarget'):
+                cursor = db.cursor()
+                cursor.execute("UPDATE ticket SET milestone=%s WHERE "
+                               "milestone=%s and status != 'closed'",
+                                (retarget_to, milestone.name))
+                self.env.log.info('Tickets associated with milestone %s '
+                                  'retargeted to %s' % 
+                                  (milestone.name, retarget_to))
         else:
             milestone.completed = 0
 
+        # don't update the milestone name until after retargetting open tickets
+        milestone.name = req.args.get('name')
         milestone.description = req.args.get('description', '')
 
         if milestone.exists:
@@ -414,7 +430,7 @@ class MilestoneModule(Component):
         db.commit()
 
         safe_name = milestone.name.replace('/', '%2F')
-        req.redirect(self.env.href.milestone(safe_name))
+        req.redirect(req.href.milestone(safe_name))
 
     def _render_confirm(self, req, db, milestone):
         req.perm.assert_permission('MILESTONE_DELETE')
@@ -423,7 +439,7 @@ class MilestoneModule(Component):
         req.hdf['milestone'] = milestone_to_hdf(self.env, db, req, milestone)
         req.hdf['milestone.mode'] = 'delete'
 
-        for idx,other in enum(Milestone.select(self.env, False, db)):
+        for idx,other in enumerate(Milestone.select(self.env, False, db)):
             if other.name == milestone.name:
                 continue
             req.hdf['milestones.%d' % idx] = other.name
@@ -433,12 +449,16 @@ class MilestoneModule(Component):
             req.perm.assert_permission('MILESTONE_MODIFY')
             req.hdf['title'] = 'Milestone %s' % milestone.name
             req.hdf['milestone.mode'] = 'edit'
+            req.hdf['milestones'] = [m.name for m in
+                                     Milestone.select(self.env)
+                                     if m.name != milestone.name]
         else:
             req.perm.assert_permission('MILESTONE_CREATE')
             req.hdf['title'] = 'New Milestone'
             req.hdf['milestone.mode'] = 'new'
 
-        from trac.util import get_date_format_hint, get_datetime_format_hint
+        from trac.util.datefmt import get_date_format_hint, \
+                                       get_datetime_format_hint
         req.hdf['milestone'] = milestone_to_hdf(self.env, db, req, milestone)
         req.hdf['milestone.date_hint'] = get_date_format_hint()
         req.hdf['milestone.datetime_hint'] = get_datetime_format_hint()
@@ -470,7 +490,7 @@ class MilestoneModule(Component):
         tickets = get_tickets_for_milestone(self.env, db, milestone.name, by)
         stats = calc_ticket_stats(tickets)
         req.hdf['milestone.stats'] = stats
-        for key, value in get_query_links(self.env, milestone.name).items():
+        for key, value in get_query_links(req, milestone.name).items():
             req.hdf['milestone.queries.' + key] = value
 
         groups = _get_groups(self.env, db, by)
@@ -490,8 +510,8 @@ class MilestoneModule(Component):
             req.hdf['%s.percent_total' % prefix] = percent_total * 100
             stats = calc_ticket_stats(group_tickets)
             req.hdf[prefix] = stats
-            for key, value in get_query_links(self.env, milestone.name,
-                                              by, group).items():
+            for key, value in \
+                    get_query_links(req, milestone.name, by, group).items():
                 req.hdf['%s.queries.%s' % (prefix, key)] = value
             group_no += 1
         req.hdf['milestone.stats.max_percent_total'] = max_percent_total * 100
@@ -505,5 +525,5 @@ class MilestoneModule(Component):
         yield ('milestone', self._format_link)
 
     def _format_link(self, formatter, ns, name, label):
-        return '<a class="milestone" href="%s">%s</a>' \
-               % (formatter.href.milestone(name), label)
+        return html.A(label, href=formatter.href.milestone(name),
+                      class_='milestone')
