@@ -1,4 +1,4 @@
-# -*- coding: iso-8859-1 -*-
+# -*- coding: utf-8 -*-
 #
 # Copyright (C) 2004-2005 Edgewall Software
 # Copyright (C) 2004 Francois Harvey <fharvey@securiweb.net>
@@ -7,26 +7,39 @@
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.com/license.html.
+# are also available at http://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://projects.edgewall.com/trac/.
+# history and logs, available at http://trac.edgewall.org/log/.
 #
 # Author: Francois Harvey <fharvey@securiweb.net>
 #         Matthew Good <trac@matt-good.net>
 
-from __future__ import generators
+from trac.config import Option
+from trac.core import *
 from trac.versioncontrol import Authorizer
 
+
+class SvnAuthzOptions(Component):
+
+    authz_file = Option('trac', 'authz_file', '',
+        """Path to Subversion
+        [http://svnbook.red-bean.com/en/1.1/ch06s04.html#svn-ch-6-sect-4.4.2 authorization (authz) file]
+        """)
+
+    authz_module_name = Option('trac', 'authz_module_name', '',
+        """The module prefix used in the authz_file.""")
+
+
 def SubversionAuthorizer(env, authname):
-    authz_file = env.config.get('trac','authz_file')    
+    authz_file = env.config.get('trac', 'authz_file')
     if not authz_file:
         return Authorizer()
 
-    module_name = env.config.get('trac','authz_module_name','')
-    db = env.get_db_cnx()
-    return RealSubversionAuthorizer(db, authname, module_name, authz_file)
+    module_name = env.config.get('trac', 'authz_module_name')
+    repos = env.get_repository()
+    return RealSubversionAuthorizer(repos, authname, module_name, authz_file)
 
 def parent_iter(path):
     path = path.strip('/')
@@ -44,14 +57,15 @@ def parent_iter(path):
         idx = path.rfind('/')
         path = path[:idx + 1]
 
+
 class RealSubversionAuthorizer(Authorizer):
 
     auth_name = ''
     module_name = ''
     conf_authz = None
 
-    def __init__(self, db, auth_name, module_name, cfg_file, cfg_fp=None):
-        self.db = db
+    def __init__(self, repos, auth_name, module_name, cfg_file, cfg_fp=None):
+        self.repos = repos
         self.auth_name = auth_name
         self.module_name = module_name
                                 
@@ -80,10 +94,9 @@ class RealSubversionAuthorizer(Authorizer):
         return 0
 
     def has_permission_for_changeset(self, rev):
-        cursor = self.db.cursor()
-        cursor.execute("SELECT path FROM node_change WHERE rev=%s", (rev,))
-        for row in cursor:
-            if self.has_permission(row[0]):
+        changeset = self.repos.get_changeset(rev)
+        for path,_,_,_,_ in changeset.get_changes():
+            if self.has_permission(path):
                 return 1
         return 0
 
@@ -142,4 +155,3 @@ class RealSubversionAuthorizer(Authorizer):
         if self.conf_authz.has_option(section, subject):
             return 'r' in self.conf_authz.get(section, subject)
         return None
-
